@@ -1,4 +1,4 @@
-// Copyright 2018 MaidSafe.net limited.
+// Copyright 2021 MaidSafe.net limited.
 //
 // This SAFE Network Software is licensed to you under The General Public License (GPL), version 3.
 // Unless required by applicable law or agreed to in writing, the SAFE Network Software distributed
@@ -8,52 +8,42 @@
 
 //! Basic example usage of a `SelfEncryptor`.
 
-// For explanation of lint checks, run `rustc -W help` or see
-// https://github.com/maidsafe/QA/blob/master/Documentation/Rust%20Lint%20Checks.md
+// For quick_error
+#![recursion_limit = "256"]
+#![doc(
+    html_logo_url = "https://raw.githubusercontent.com/maidsafe/QA/master/Images/maidsafe_logo.png",
+    html_favicon_url = "https://maidsafe.net/img/favicon.ico",
+    test(attr(deny(warnings)))
+)]
+// Forbid some very bad patterns. Forbid is stronger than `deny`, preventing us from suppressing the
+// lint with `#[allow(...)]` et-all.
 #![forbid(
     arithmetic_overflow,
     mutable_transmutes,
     no_mangle_const_items,
-    unknown_crate_types
-)]
-#![deny(
-    bad_style,
-    deprecated,
-    improper_ctypes,
-    missing_docs,
-    non_shorthand_field_patterns,
-    overflowing_literals,
-    stable_features,
-    unconditional_recursion,
-    unknown_lints,
-    unsafe_code,
-    unused,
-    unused_allocation,
-    unused_attributes,
-    unused_comparisons,
-    unused_features,
-    unused_parens,
-    while_true,
-    warnings
+    unknown_crate_types,
+    unsafe_code
 )]
+// Turn on some additional warnings to encourage good style.
 #![warn(
+    missing_debug_implementations,
+    missing_docs,
     trivial_casts,
     trivial_numeric_casts,
+    unreachable_pub,
     unused_extern_crates,
     unused_import_braces,
     unused_qualifications,
-    unused_results
-)]
-#![allow(
-    box_pointers,
-    missing_copy_implementations,
-    missing_debug_implementations,
-    variant_size_differences
+    unused_results,
+    clippy::unicode_not_nfc
 )]
 
-use async_trait::async_trait;
+use bytes::Bytes;
 use docopt::Docopt;
-use self_encryption::{self, test_helpers, DataMap, SelfEncryptionError, SelfEncryptor, Storage};
+use rayon::prelude::*;
+use self_encryption::{
+    self, decrypt_full_set, encrypt, test_helpers, EncryptedChunk, Error, Result, SecretKey,
+};
 use serde::Deserialize;
 use std::{
     env,
@@ -62,8 +52,9 @@ use std::{
     io::{Read, Write},
     path::PathBuf,
     string::String,
+    sync::Arc,
 };
-use tiny_keccak::{Hasher, Sha3};
+use xor_name::XorName;
 
 #[rustfmt::skip]
 static USAGE: &str = "
@@ -90,63 +81,43 @@ fn to_hex(ch: u8) -> String {
     fmt::format(format_args!("{:02x}", ch))
 }
 
-fn file_name(name: &[u8]) -> String {
+fn file_name(name: XorName) -> String {
     let mut string = String::new();
-    for ch in name {
-        string.push_str(&to_hex(*ch));
+    for ch in name.0 {
+        string.push_str(&to_hex(ch));
     }
     string
 }
 
 #[derive(Clone)]
 struct DiskBasedStorage {
-    pub storage_path: String,
+    pub(crate) storage_path: String,
 }
 
 impl DiskBasedStorage {
-    fn calculate_path(&self, name: &[u8]) -> PathBuf {
+    fn calculate_path(&self, name: XorName) -> PathBuf {
         let mut path = PathBuf::from(self.storage_path.clone());
         path.push(file_name(name));
         path
     }
-}
 
-#[async_trait]
-impl Storage for DiskBasedStorage {
-    async fn get(&mut self, name: &[u8]) -> Result<Vec<u8>, SelfEncryptionError> {
+    fn get(&self, name: XorName) -> Result<Bytes, Error> {
         let path = self.calculate_path(name);
         let mut file = File::open(&path)?;
         let mut data = Vec::new();
         let _ = file.read_to_end(&mut data);
-
-        Ok(data)
+        Ok(Bytes::from(data))
     }
 
-    async fn put(&mut self, name: Vec<u8>, data: Vec<u8>) -> Result<(), SelfEncryptionError> {
-        let path = self.calculate_path(&name);
+    fn put(&self, name: XorName, data: Bytes) -> Result<()> {
+        let path = self.calculate_path(name);
         let mut file = File::create(&path)?;
-
         file.write_all(&data[..])
             .map(|_| {
                 println!("Chunk written to {:?}", path);
             })
             .map_err(From::from)
     }
-
-    async fn delete(&mut self, name: &[u8]) -> Result<(), SelfEncryptionError> {
-        let path = self.calculate_path(&name);
-        fs::remove_file(path)?;
-
-        Ok(())
-    }
-
-    async fn generate_address(&self, data: &[u8]) -> Result<Vec<u8>, SelfEncryptionError> {
-        let mut hasher = Sha3::v256();
-        let mut output = [0; 32];
-        hasher.update(&data);
-        hasher.finalize(&mut output);
-        Ok(output.to_vec())
-    }
 }
 
 // use tokio to enable an async main func
@@ -162,59 +133,47 @@ async fn main() {
     let mut chunk_store_dir = env::temp_dir();
     chunk_store_dir.push("chunk_store_test/");
     let _ = fs::create_dir(chunk_store_dir.clone());
-    let mut storage = DiskBasedStorage {
-        storage_path: chunk_store_dir.to_str().unwrap().to_owned(),
-    };
+    let storage_path = chunk_store_dir.to_str().unwrap().to_owned();
+    let storage = Arc::new(DiskBasedStorage { storage_path });
 
-    let mut data_map_file = chunk_store_dir;
-    data_map_file.push("data_map");
+    let mut secret_key_file = chunk_store_dir;
+    secret_key_file.push("secret_key");
 
     if args.flag_encrypt && args.arg_target.is_some() {
         if let Ok(mut file) = File::open(args.arg_target.clone().unwrap()) {
-            match file.metadata() {
-                Ok(metadata) => {
-                    if metadata.len() > self_encryption::MAX_FILE_SIZE as u64 {
-                        return println!(
-                            "File size too large {} is greater than 1GB",
-                            metadata.len()
-                        );
-                    }
-                }
-                Err(error) => return println!("{}", error.to_string()),
-            }
-
             let mut data = Vec::new();
             match file.read_to_end(&mut data) {
                 Ok(_) => (),
                 Err(error) => return println!("{}", error.to_string()),
             }
 
-            let se = SelfEncryptor::new(storage, DataMap::None)
-                .expect("Encryptor construction shouldn't fail.");
-            se.write(&data, 0)
-                .await
-                .expect("Writing to encryptor shouldn't fail.");
-            let (data_map, old_storage) =
-                se.close().await.expect("Closing encryptor shouldn't fail.");
-            storage = old_storage;
+            let (secret_key, encrypted_chunks) = encrypt(Bytes::from(data)).unwrap();
 
-            match File::create(data_map_file.clone()) {
+            let result = encrypted_chunks
+                .par_iter()
+                .map(|c| (c, storage.clone()))
+                .map(|(c, store)| store.put(XorName::from_content(&c.content), c.content.clone()))
+                .collect::<Vec<_>>();
+
+            assert!(result.iter().all(|r| r.is_ok()));
+
+            match File::create(secret_key_file.clone()) {
                 Ok(mut file) => {
-                    let encoded = test_helpers::serialise(&data_map).unwrap();
+                    let encoded = test_helpers::serialise(&secret_key).unwrap();
                     match file.write_all(&encoded[..]) {
-                        Ok(_) => println!("Data map written to {:?}", data_map_file),
+                        Ok(_) => println!("Secret key written to {:?}", secret_key_file),
                         Err(error) => {
                             println!(
-                                "Failed to write data map to {:?} - {:?}",
-                                data_map_file, error
+                                "Failed to write secret key to {:?} - {:?}",
+                                secret_key_file, error
                             );
                         }
                     }
                 }
                 Err(error) => {
                     println!(
-                        "Failed to create data map at {:?} - {:?}",
-                        data_map_file, error
+                        "Failed to create secret key at {:?} - {:?}",
+                        secret_key_file, error
                     );
                 }
             }
@@ -227,37 +186,49 @@ async fn main() {
         if let Ok(mut file) = File::open(args.arg_target.clone().unwrap()) {
             let mut data = Vec::new();
             let _ = file.read_to_end(&mut data).unwrap();
+            match test_helpers::deserialise::<SecretKey>(&data) {
+                Ok(secret_key) => {
+                    let (keys, encrypted_chunks) = secret_key
+                        .keys()
+                        .par_iter()
+                        .map(|key| {
+                            Ok::<(_, _), Error>((
+                                key.clone(),
+                                EncryptedChunk {
+                                    index: key.index,
+                                    content: storage.get(key.dst_hash)?,
+                                },
+                            ))
+                        })
+                        .collect::<Vec<_>>()
+                        .into_iter()
+                        .flatten()
+                        .fold((vec![], vec![]), |(mut keys, mut chunks), (key, chunk)| {
+                            keys.push(key);
+                            chunks.push(chunk);
+                            (keys, chunks)
+                        });
 
-            match test_helpers::deserialise::<DataMap>(&data) {
-                Ok(data_map) => {
-                    let se = SelfEncryptor::new(storage, data_map)
-                        .expect("Encryptor construction shouldn't fail.");
-                    let length = se.len().await;
                     if let Ok(mut file) = File::create(args.arg_destination.clone().unwrap()) {
-                        let content = se
-                            .read(0, length)
-                            .await
-                            .expect("Reading from encryptor shouldn't fail.");
+                        let content =
+                            decrypt_full_set(&SecretKey::new(keys), encrypted_chunks.as_ref())
+                                .unwrap();
                         match file.write_all(&content[..]) {
                             Err(error) => println!("File write failed - {:?}", error),
-                            Ok(_) => println!(
-                                "File decrypted to {:?}",
-                                args.arg_destination.clone().unwrap()
-                            ),
+                            Ok(_) => {
+                                println!("File decrypted to {:?}", args.arg_destination.unwrap())
+                            }
                         };
                     } else {
-                        println!(
-                            "Failed to create {}",
-                            (args.arg_destination.clone().unwrap())
-                        );
+                        println!("Failed to create {}", (args.arg_destination.unwrap()));
                     }
                 }
                 Err(_) => {
-                    println!("Failed to parse data map - possible corruption");
+                    println!("Failed to parse secret key - possible corruption");
                 }
             }
         } else {
-            println!("Failed to open data map at {:?}", data_map_file);
+            println!("Failed to open secret key at {:?}", secret_key_file);
         }
     }
 }
